# Szkolenie GNU/Linux

## Podstawy obsługi systemu plików

### Gdzie jestem?

```bash
pwd
ls
ls -l
ls -al
ls -alh
ls -alh --group-directories-first
ls /
ls /home/moja_nazwa_uzytkownika
```

### Dokąd zmierzam?

```bash
mkdir moje
ls
cd moje
ls
cd ..
ls
cd moje
ls
touch plik
ls
ls -l
rm plik
ls
touch plik
cp plik kopia_pliku
cp plik ~/kopia_pliku
ls ~/
rm kopia_pliku
ls
mv ~/kopia_pliku .
```

### Uprawnienia - co wolno mi zrobić?

```bash
ls -l
chmod +x plik
chmod -r plik
ls -l
groups
chown ja:moja_grupa plik
```

### Edycja tekstu

```bash
vim plik_tekstowy
# demo vima...
# Polecamy ściągawkę, wystarczy wyszukać "vim cheatsheet"
```

## Programy i aplikacje

### Demo C++

- argumenty
- kod wyjścia
- zmienne środowiskowe

### Podstawowe narzędzia

```bash
vim przyklad.txt # tworzymy przykladowy plik
echo przyklad.txt
echo "echo, echo, echo..."
touch --help
man touch
cat przyklad.txt
bat przyklad.txt # bat wymaga instalacji
wc przyklad.txt
wc -l przyklad.txt
grep "a" przyklad.txt
grep "[[:digit:]]" przyklad.txt
head przyklad.txt
head -n 5 przyklad.txt
tail przyklad.txt
sed '1 a To jest nowa linijka' przyklad.txt
sed -i '1 a To jest nowa linijka' przyklad.txt
```

### Archiwa

```bash
cp przyklad.txt przyklad2.txt
mkdir katalog
mv przyklad.txt katalog/
mv przyklad2.txt katalog/
cp -r katalog/ katalog2/
tar -czvf katalog_arch.tar katalog2
rm -r katalog2
ls
tar -xzvf katalog_arch.tar
```

### "Smaczki"

- klawisz tab
- historia komend: strzałki ^v
- historia komend: ctrl+r
- przerywanie wykonywania programu: ctrl+c
- znak specjalny *
- kopiowanie i wklejanie

## Złożone zadania - skrypty, potoki

### Zmienne

```bash
#!/bin/bash

# Zmienne
x="zmienna x"
echo $x
```

### Warunek logiczny `if`

```bash
if [ "$x"="zmienna x" ]; then
  echo 'x = zmienna x'
else
  echo $x
fi
```

### Pętla `for`

```bash
for i in {1..5}; do
  echo i
done
```

### Argumenty

```bash
arg1=$1
if [ -n $arg1 ]; then
  echo Podano co najmniej jeden argument
fi
if [ -d $arg1 ]; then
  for file in "$arg1"/*; do
    echo $arg1
  done
fi
```

### Arytmetyka

```bash
if [ $1 -le 1 ]; then
  echo 1
  exit 0
fi

a=1
b=1
end=$(($1-2))
for (( i=0; i<=$end; i++ )); do
  fibo=$((a+b))
  a=$b
  b=$fibo
done
echo $fibo
```

### Potoki i przekierowania

```bash
cat przyklad.txt | grep -i slowo
cat /proc/cpuinfo | grep processor | wc -l | tr -d '\n' || (echo 'oops :<' && exit 1)
cat /proc/cpuinfo | grep processor | wc -l | tr -d '\n' > result.txt || (echo 'oops :<' && exit 1)
cat /proc/cpuinfo | grep processor | wc -l | tr -d '\n' &> result.txt || (echo 'oops :<' && exit 1)
grep processor < /proc/cpuinfo | wc -l | tr -d '\n' &> result.txt || (echo 'oops :<' && exit 1)
```

## Praca z systemami zdalnymi

### `ssh`

```bash
ssh user@server
ssh jgalecki@hpc.icm.edu.pl
```

- katalog `~/.ssh`
- `~/.ssh/config`

### `scp`

Tak jak `cp`, tylko przez `ssh`:

- `scp skąd dokąd`
- `scp plik_lokalny user@server:ścieżka`

```bash
scp przyklad.txt jgalecki@hpc.icm.edu.pl:/home/jgalecki/temp/
scp jgalecki@hpc.icm.edu.pl:/home/jgalecki/temp/przyklad.txt ~/temp/
```

Dodatkowe narzędzia:

- `rsync`
- `sftp`

## Demonstracja: workflow HPC

Na żywo!

## Arkusz zadań

### Zadanie 1

1. Stwórz w swoim katalogu domowym katalog o nazwie `test-dir`
2. W katalogu `test-dir` przy pomocy edytora `vim` stwórz plik `imie.txt` i wpisz do niego swoje imię
3. Skopiuj plik `imie.txt`, nadaj kopii nazwę `kolor.txt`
4. Zedytuj plik `kolor.txt` tak, aby zawierał nazwę Twojego ulubionego koloru
5. Przy pomocy komendy `chmod` oraz specjalnego znaku `*` odbierz możliwość modyfikacji wszystkich plików o rozszerzeniu `.txt` w katalogu stworzonym w punkcie 1.
6. Zweryfikuj poprawność wykonania punktu 5. za pomocą komendy `ls` z odpowiednią flagą

### Zadanie 2

Napisz skrypt, który:
1. Przyjmuje 1 argument, będący ścieżką do katalogu (sprawdź, czy podany argument faktycznie jest katalogiem i w razie niepowodzenia wydrukuj odpowiedni komunikat)
2. Tworzy w podany katalogu pliki `f1.txt` - `f20.txt`
3. Do każdego pliku wpisuje wartość odpowiedniej potęgi dwójki, tzn. `f1.txt` zawiera liczbę 2, `f2.txt` zawiera liczbę 4, `f3.txt` zawiera liczbę 8, itd. (ten krok możesz połączyć z poprzednim)
4. Kompresuje katalog do archiwum przy użyciu wybranego algorytmu kompresji

### Zadanie 3
1. Stwórz lokalnie katalog `data`, dodaj do niego kilka wybranych plików
2. Skopiuj katalog `data` na maszynę zdalną `hpc.icm.edu.pl`
3. Połącz się z maszyną zdalną `hpc.icm.edu.pl`
4. Zedytuj wrzucone przed chwilą pliki
5. Zakończ sesję zdalną
6. Skopiuj pliki z maszyny zdalnej na lokalną

